@@ -443,8 +443,19 @@ nsContentSink::ProcessHTTPHeaders(nsIChannel* aChannel)
   nsresult rv = httpchannel->GetResponseHeader(NS_LITERAL_CSTRING("link"),
                                                linkHeader);
   if (NS_SUCCEEDED(rv) && !linkHeader.IsEmpty()) {
-    ProcessHeaderData(nsGkAtoms::link,
-                      NS_ConvertASCIItoUTF16(linkHeader));
+    mDocument->SetHeaderData(nsGkAtoms::link,
+                             NS_ConvertASCIItoUTF16(linkHeader));
+
+    NS_ASSERTION(!mProcessLinkHeaderEvent.get(),
+                 "Already dispatched an event?");
+
+    mProcessLinkHeaderEvent =
+      new nsNonOwningRunnableMethod<nsContentSink>(this,
+                                           &nsContentSink::DoProcessLinkHeader);
+    rv = NS_DispatchToCurrentThread(mProcessLinkHeaderEvent.get());
+    if (NS_FAILED(rv)) {
+      mProcessLinkHeaderEvent.Forget();
+    }
   }
   
   return NS_OK;
@@ -539,6 +550,14 @@ nsContentSink::ProcessHeaderData(nsIAtom* aHeader, const nsAString& aValue,
 }
 
 
+void
+nsContentSink::DoProcessLinkHeader()
+{
+  nsAutoString value;
+  mDocument->GetHeaderData(nsGkAtoms::link, value);
+  ProcessLinkHeader(nsnull, value);
+}
+
 static const PRUnichar kSemiCh = PRUnichar(';');
 static const PRUnichar kCommaCh = PRUnichar(',');
 static const PRUnichar kEqualsCh = PRUnichar('=');
@@ -1711,6 +1730,12 @@ nsContentSink::WillBuildModelImpl()
   }
 
   mScrolledToRefAlready = PR_FALSE;
+
+  if (mProcessLinkHeaderEvent.get()) {
+    mProcessLinkHeaderEvent.Revoke();
+
+    DoProcessLinkHeader();
+  }
 }
 
 void
@@ -1735,7 +1760,7 @@ nsContentSink::ReadyToCallDidBuildModelImpl(PRBool aTerminated)
 {
   if (!mDidGetReadyToCallDidBuildModelCall) {
     if (mScriptLoader) {
-      mScriptLoader->EndDeferringScripts(aTerminated);
+      mScriptLoader->ParsingComplete(aTerminated);
     }
   }
 
