@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Gary Kwong
+ *                 Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +36,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452498-101.js';
+var gTestfile = 'regress-472528-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 472528;
+var summary = 'Do not assert: !js_IsActiveWithOrBlock(cx, fp->scopeChain, 0)';
 var actual = '';
 var expect = '';
 
@@ -53,10 +54,19 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// ------- Comment #101 From Gary Kwong [:nth10sd]
+  jit(true);
 
-  uneval(function(){with({functional: []}){x5, y = this;const y }});
-// Assertion failure: strcmp(rval, with_cookie) == 0, at ../jsopcode.cpp:2567
+  try
+  {
+    for (var i = 0; i < 4; ++i) {
+      for (let j = 0; j < 2; ++j) { }
+      let (x) (function(){});
+    }
+  }
+  catch(ex)
+  {
+  }
+  jit(false);
 
   reportCompare(expect, actual, summary);
 
