@@ -59,36 +59,27 @@ function LOG(aMsg) {
 
 // If there's no location registered for the profile directory, register one now.
 var dirSvc = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
-var profileDir = null;
-try {
- profileDir = dirSvc.get(NS_APP_USER_PROFILE_50_DIR, Ci.nsIFile);
-} catch (e) {}
-if (!profileDir) {
- // Register our own provider for the profile directory.
- // It will simply return the current directory.
- var provider = {
-   getFile: function(prop, persistent) {
-     persistent.value = true;
-     if (prop == NS_APP_USER_PROFILE_50_DIR) {
-       return dirSvc.get("CurProcD", Ci.nsIFile);
-     }
-     if (prop == NS_APP_HISTORY_50_FILE) {
-       var histFile = dirSvc.get("CurProcD", Ci.nsIFile);
-       histFile.append("history.dat");
-       return histFile;
-     }
-     throw Cr.NS_ERROR_FAILURE;
-   },
-   QueryInterface: function(iid) {
-     if (iid.equals(Ci.nsIDirectoryServiceProvider) ||
-         iid.equals(Ci.nsISupports)) {
-       return this;
-     }
-     throw Cr.NS_ERROR_NO_INTERFACE;
-   }
- };
- dirSvc.QueryInterface(Ci.nsIDirectoryService).registerProvider(provider);
-}
+var profileDir = do_get_profile();
+
+var provider = {
+  getFile: function(prop, persistent) {
+    persistent.value = true;
+    if (prop == NS_APP_HISTORY_50_FILE) {
+      var histFile = profileDir.clone();
+      histFile.append("history.dat");
+      return histFile;
+    }
+    throw Cr.NS_ERROR_FAILURE;
+  },
+  QueryInterface: function(iid) {
+    if (iid.equals(Ci.nsIDirectoryServiceProvider) ||
+        iid.equals(Ci.nsISupports)) {
+      return this;
+    }
+    throw Cr.NS_ERROR_NO_INTERFACE;
+  }
+};
+dirSvc.QueryInterface(Ci.nsIDirectoryService).registerProvider(provider);
 
 var iosvc = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
 
